@@ -4,7 +4,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 =head1 NAME
 
@@ -12,7 +12,7 @@ Crypt::LE - Let's Encrypt API interfacing module.
 
 =head1 VERSION
 
-Version 0.11
+Version 0.12
 
 =head1 SYNOPSIS
 
@@ -43,21 +43,24 @@ Version 0.11
 =head1 DESCRIPTION
 
 Crypt::LE provides the functionality necessary to use Let's Encrypt API and generate free SSL certificates for your domains. It can also
-be used to generate RSA keys and Certificate Signing Requests or to revoke previously issued certificates.
+be used to generate RSA keys and Certificate Signing Requests or to revoke previously issued certificates. Crypt::LE is shipped with a 
+self-sufficient client for obtaining SSL certificates - le.pl.
+
+B<Provided client supports 'http' and 'dns' domain verification out of the box.>
 
 Crypt::LE can be easily extended with custom plugins to handle Let's Encrypt challenges. See L<Crypt::LE::Challenge::Simple> module
 for an example of a challenge-handling plugin.
 
-Crypt::LE is shipped with a self-sufficient client for obtaining SSL certificates - le.pl. Usage example:
+Basic usage:
 
 B<le.pl --key account.key --csr domain.csr --csr-key domain.key --crt domain.crt --domains "www.domain.ext,domain.ext" --generate-missing>
 
 That will generate an account key and a CSR (plus key) if they are missing. If any of those files exist, they will just be loaded, so it is safe to re-run
 the client. Run le.pl without any parameters or with C<--help> to see more details and usage examples.
 
-The provided client also supports challenge-handling plugins, such as L<Crypt::LE::Challenge::Simple>. In addition, it supports completion-handling
-plugins, such as L<Crypt::LE::Complete::Simple>. You can easily handle challenges and trigger specific actions when your certificate gets issued
-by using those modules as templates, without modifying the client code. You can also pass custom parameters to your modules from le.pl command line:
+In addition to challenge-handling plugins, the client also supports completion-handling plugins, such as L<Crypt::LE::Complete::Simple>. You can easily 
+handle challenges and trigger specific actions when your certificate gets issued by using those modules as templates, without modifying the client code.
+You can also pass custom parameters to your modules from le.pl command line:
 
 B<le.pl ... --handle-with Crypt::LE::Challenge::Simple --handle-params '{"key1": 1, "key2": "one"}'>
  
@@ -68,7 +71,7 @@ The parameters don't have to be put directly in the command line, you could also
 B<le.pl ... --complete-params complete.json>
  
 Crypt::LE::Challenge:: and Crypt::LE::Complete:: namespaces are suggested for new plugins.
- 
+
 =head1 EXPORT
 
 Crypt::LE does not export anything by default, but allows you to import the following constants:
@@ -93,6 +96,9 @@ DATA_MISMATCH
 =item *
 ALREADY_DONE
 
+=item *
+AUTH_ERROR
+
 =item *
 ERROR
 
@@ -147,11 +153,12 @@ use constant ERROR                  => 500;
 use constant SUCCESS                => 200;
 use constant CREATED                => 201;
 use constant ACCEPTED               => 202;
+use constant AUTH_ERROR             => 403;
 use constant ALREADY_DONE           => 409;
 
 use constant NID_subject_alt_name   => 85;
 
-our @EXPORT_OK = (qw<OK READ_ERROR LOAD_ERROR INVALID_DATA DATA_MISMATCH ERROR ALREADY_DONE>);
+our @EXPORT_OK = (qw<OK READ_ERROR LOAD_ERROR INVALID_DATA DATA_MISMATCH ERROR AUTH_ERROR ALREADY_DONE>);
 our %EXPORT_TAGS = ( errors => [ @EXPORT_OK ] );
 
 my $header = 'replay-nonce';
@@ -212,6 +219,10 @@ Enables automatic retrieval of the resource directory (required for normal API p
 Specifies the time in seconds to wait before Let's Encrypt servers are checked for the challenge verification results again. By default set to 2 seconds.
 Non-integer values are supported (so for example you can set it to 1.5 if you like).
 
+=item C<logger>
+
+Logger instance to use for debug messages. If not given, the messages will be printed to STDOUT.
+
 =back
 
 Returns: L<Crypt::LE> object.
@@ -236,6 +247,8 @@ sub new {
     $self->{ua} = HTTP::Tiny->new( agent => $self->{ua} || __PACKAGE__ . " v$VERSION", verify_SSL => 1 );
     # Init server
     $self->{server} ||= $self->{live} ? 'acme-v01.api.letsencrypt.org' : 'acme-staging.api.letsencrypt.org';
+    # Init logger
+    $self->{logger} = $params{logger} if ($params{logger} and blessed $params{logger});
     bless $self, $class;
 }
 
@@ -606,6 +619,10 @@ The challenge token
  
 The account key fingerprint
 
+=item C<logger>
+ 
+Logger object.
+
 =back
 
 The type of the challenge accepted is optional and it is 'http' by default. The following values are currently available: 'http', 'tls', 'dns'. 
@@ -662,7 +679,7 @@ sub accept_challenge {
             push @domains_failed, $domain;
         }
         my $rv;
-        my $callback_data = { domain => $domain, token => $self->{challenges}->{$domain}->{$type}->{token}, fingerprint => $self->{fingerprint} };
+        my $callback_data = { domain => $domain, token => $self->{challenges}->{$domain}->{$type}->{token}, fingerprint => $self->{fingerprint}, logger => $self->{logger} };
         eval {
             $rv = $mod_callback ? $cb->$handler($callback_data, $params) : &$cb($callback_data, $params); 
         };
@@ -709,6 +726,14 @@ The account key fingerprint
  
 Set to 1 if the domain has been verified successfully or set to 0 otherwise.
 
+=item C<error>
+ 
+Error message returned for domain on verification failure.
+
+=item C<logger>
+ 
+Logger object.
+
 =back
 
 The type of the challenge accepted is optional and it is 'http' by default. The following values are currently available: 'http', 'tls', 'dns'.
@@ -787,7 +812,15 @@ sub verify_challenge {
         }
         if ($cb) {
             my $rv;
-            my $callback_data = { domain => $domain, token => $self->{challenges}->{$domain}->{$type}->{token}, fingerprint => $self->{fingerprint}, valid => $validated };
+            my $error = (ref $content eq 'HASH' and $content->{error} and $content->{error}->{detail}) ? $content->{error}->{detail} : '';
+            my $callback_data = { 
+                                    domain => $domain, 
+                                    token => $self->{challenges}->{$domain}->{$type}->{token},
+                                    fingerprint => $self->{fingerprint}, 
+                                    valid => $validated, 
+                                    error => $error,
+                                    logger => $self->{logger},
+                                };
             eval {
                 $rv = $mod_callback ? $cb->$handler($callback_data, $params) : &$cb($callback_data, $params); 
             };
@@ -818,7 +851,7 @@ sub verify_challenge {
 
 Requests the certificate for your CSR.
 
-Returns: OK | ERROR.
+Returns: OK | AUTH_ERROR | ERROR.
 
 =cut
 
@@ -833,7 +866,7 @@ sub request_certificate {
         $self->{issuer_url} = ($self->{links} and $self->{links}->{up}) ? $self->{links}->{up} : undef;
         return $self->_status(OK, "Domain certificate has been received." . ($self->{issuer_url} ? " Issuer's certificate can be found at: $self->{issuer_url}" : ""));
     }
-    return $self->_status(ERROR, $content);
+    return $self->_status($status == AUTH_ERROR ? AUTH_ERROR : ERROR, $content);
 }
 
 =head2 request_issuer_certificate()
@@ -1026,7 +1059,7 @@ sub _request {
     my $self = shift;
     my ($url, $payload) = @_;
     my $resp = $payload ? $self->{ua}->post($url, { content => $self->_jws($payload) }) : $self->{ua}->get($url);
-    my $slurp = ($resp->{headers}->{'content-type'} and $resp->{headers}->{'content-type'}=~/^application\/json/) ? 0 : 1;
+    my $slurp = ($resp->{headers}->{'content-type'} and $resp->{headers}->{'content-type'}=~/^application\/(?:problem\+)?json/) ? 0 : 1;
     $self->_debug($slurp ? $resp->{headers} : $resp);
     $self->{nonce} = $resp->{headers}->{$header};
     my ($status, $rv) = ($resp->{status}, $resp->{content});
@@ -1087,9 +1120,9 @@ sub _debug {
     return unless $self->{debug};
     foreach (@_) {
         if (!ref $_) {
-            print "$_\n";
+            $self->{logger} ? $self->{logger}->debug($_) : print "$_\n";
         } elsif ($self->{debug} > 1) {
-            print Dumper($_);
+            $self->{logger} ? $self->{logger}->debug(Dumper($_)) : print Dumper($_);
         }
     }
 }
@@ -1100,7 +1133,11 @@ sub _status {
     if ($code == OK) {
         undef $self->{error};
     } else {
-        $self->{error} = $data||$code;
+        if (ref $data eq 'HASH' and $data->{error}) {
+            $self->{error} = $data->{error};
+        } else {
+            $self->{error} = $data||$code;
+        }
     }
     $self->_debug($data) if $data;
     return $code;
@@ -1169,6 +1206,16 @@ L<http://cpanratings.perl.org/d/Crypt-LE>
 
 L<http://search.cpan.org/dist/Crypt-LE/>
 
+=item * Project homepage
+
+L<https://ZeroSSL.com/>
+
+=item * Company homepage
+
+L<https://Do-Know.com/>
+
+
+
 =back
 
 =head1 LICENSE AND COPYRIGHT
