@@ -1,9 +1,8 @@
 package Crypt::LE::Complete::Simple;
-use Data::Dumper;
 use strict;
 use warnings;
 
-our $VERSION = '0.07';
+our $VERSION = '0.12';
 
 =head1 NAME
 
@@ -28,8 +27,10 @@ sub new { bless {}, shift }
 sub complete {
     my $self = shift;
     my ($data, $params) = @_;
-    if ($params) {
-        print "Parameters passed: " . Dumper($params);
+    # You can use external logger if it has been provided.
+    if ($data->{logger}) {
+        # NB: The list of domains the certificate is issued for is in $data->{domains} (array ref).
+        $data->{logger}->info("Handling process completion for " . join(', ', @{$data->{domains}}) . " with " .  __PACKAGE__);
     }
     print "Domain Certificate '$data->{certificate_file}':\n$data->{certificate}\n";
     print "Issuer's Certificate:\n$data->{issuer}\n";
